package com.example.mohmou.networkclient;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.IBinder;
import android.os.Looper;
import android.os.Message;
import android.os.Process;
import android.support.v4.app.NotificationCompat;
import android.widget.Toast;

import java.io.BufferedInputStream;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.Scanner;

public class EchoService extends Service {

    private Looper mServiceLooper;
    private CommunicationHandler mServiceHandler;


    public void onCreate() {
        HandlerThread thread = new HandlerThread("TheServiceWorkerThread",
                                                    Process.THREAD_PRIORITY_BACKGROUND );
        Toast.makeText(this, "In Service", Toast.LENGTH_SHORT).show();

        thread.start();
        mServiceLooper = thread.getLooper();
        mServiceHandler = new CommunicationHandler(mServiceLooper);
    }

    public int onStartCommand(Intent intent, int flags, int startId) {
        Toast.makeText(this, "service starting", Toast.LENGTH_SHORT).show();
        Message msg = mServiceHandler.obtainMessage();
        msg.what = intent.getExtras().getInt("what");
        msg.setData(intent.getBundleExtra("arguments"));
        mServiceHandler.sendMessage(msg);
        return Service.START_STICKY;
    }


    @Override
    public IBinder onBind(Intent intent) {
        // TODO: Return the communication channel to the service.
        //throw new UnsupportedOperationException("Not yet implemented");
        return null;
    }


    private final class CommunicationHandler extends Handler {
        private Scanner in;
        private PrintWriter out;
        private Socket socket;
        private String data;
        private String ip = "192.168.227.59";
        private Integer port = 4445;

        public CommunicationHandler(Looper looper) {
            super(looper);
        }

        public void handleMessage(Message msg) {
            switch(msg.what){

                case 1 :
                    try{
                        Toast.makeText(EchoService.this, "Opening socket", Toast.LENGTH_SHORT).show();
                        socket = new Socket(ip, port);
                        // Establish a connection with the server
                        in = new Scanner(new BufferedInputStream(socket.getInputStream()));
                        out = new PrintWriter(socket.getOutputStream(), true);
                    }catch(java.io.IOException e){
                        Toast.makeText(EchoService.this, "Unknown host", Toast.LENGTH_SHORT).show();
                        stopSelf();
                    }

                    // Got from the activity something to send to the server
                    String data =  msg.getData().getString("data");
                    out.println(data);
                    // wait to receive an acknowledgement from the server
                    String back = in.nextLine();
                    sendNotification("Sent: " + data + ", Got: " + back + ".");
                    try{
                        out.close();in.close();socket.close();stopSelf();
                    }catch(java.io.IOException e){
                        Toast.makeText(EchoService.this, "Could not close connection...", Toast.LENGTH_SHORT).show();
                        stopSelf();
                    }
                    break;

                case 2 :
                    stopSelf();
                    break;
            }


        }
    }


    @SuppressWarnings("deprecation")
    private void sendNotification(String s){
//	    String ns                                = Context.NOTIFICATION_SERVICE;
        NotificationManager mNotificationManager =
                (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);

        int icon        = R.mipmap.ic_launcher;        // icon from resources
        int mId			= 1;
        Context context = getApplicationContext();      // application Context

        CharSequence tickerText   = "Message from client";
        CharSequence contentTitle = "Phone says";
        CharSequence contentText  = s;

        NotificationCompat.Builder notification =
                new NotificationCompat.Builder(getApplicationContext())
                        .setSmallIcon(icon)
                        .setContentTitle(contentTitle)
                        .setTicker(tickerText)
                        .setContentText(contentText);


	    /* the notification will call an activity with this intent */
        Intent notifyIntent = new Intent(DisplayEcho.SEND_ACTION);
        notifyIntent.setClass(context, DisplayEcho.class);
        notifyIntent.putExtra("data", contentText);
        PendingIntent contentIntent = PendingIntent.getActivity(context, 0,
                notifyIntent,  PendingIntent.FLAG_UPDATE_CURRENT );

	    /* complete configuration and notify */
        notification.setContentIntent(contentIntent);

        // mId allows you to update the notification later on.
        mNotificationManager.notify(mId, notification.build());
    }
}

